package ru.dbotthepony.mc.otm.data

import com.google.gson.JsonDeserializationContext
import com.google.gson.JsonObject
import com.google.gson.JsonSerializationContext
import com.google.gson.JsonSyntaxException
import com.mojang.serialization.Codec
import net.minecraft.world.level.storage.loot.Serializer
import ru.dbotthepony.mc.otm.core.fromJsonStrict
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.core.toJsonStrict

class Codec2Serializer<T : Any>(val codec: Codec<T>, val embed: Boolean = true) : Serializer<T> {
	override fun serialize(p_79325_: JsonObject, p_79326_: T, p_79327_: JsonSerializationContext) {
		if (embed) {
			val json = codec.toJsonStrict(p_79326_)

			if (json !is JsonObject) {
				throw RuntimeException("It was specified to embed result from codec into target json, but codec returned ${json::class.qualifiedName}, while was expecting JsonObject")
			}

			for ((k, v) in json.entrySet()) {
				if (p_79325_.has(k)) {
					throw RuntimeException("Codec returned object with '$k' as member, which is not allowed (it was written by upstream code into provided JsonObject)")
				}

				p_79325_[k] = v
			}
		} else {
			p_79325_["value"] = codec.toJsonStrict(p_79326_)
		}
	}

	override fun deserialize(p_79323_: JsonObject, p_79324_: JsonDeserializationContext): T {
		return codec.fromJsonStrict(p_79323_["value"] ?: throw JsonSyntaxException("Missing 'value' element"))
	}
}
